/*
 *
 * This file is part of the iText (R) project. Copyright (c) 1998-2011 1T3XT BVBA Authors: Balder Van Camp, et al.
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU Affero General
 * Public License version 3 as published by the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY
 * 1T3XT, 1T3XT DISCLAIMS THE WARRANTY OF NON INFRINGEMENT OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
 * details. You should have received a copy of the GNU Affero General Public License along with this program; if not,
 * see http://www.gnu.org/licenses or write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA, 02110-1301 USA, or download the license from the following URL: http://itextpdf.com/terms-of-use/
 *
 * The interactive user interfaces in modified source and object code versions of this program must display Appropriate
 * Legal Notices, as required under Section 5 of the GNU Affero General Public License.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License, a covered work must retain the producer
 * line in every PDF that is created or manipulated using iText.
 *
 * You can be released from the requirements of the license by purchasing a commercial license. Buying such a license is
 * mandatory as soon as you develop commercial activities involving the iText software without disclosing the source
 * code of your own applications. These activities include: offering paid services to customers as an ASP, serving PDFs
 * on the fly in a web application, shipping iText with a closed source product.
 *
 * For more information, please contact iText Software Corp. at this address: sales@itextpdf.com
 */
package com.itextpdf.maven.itextdoc;

import java.io.File;
import java.io.IOException;
import java.nio.charset.Charset;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;

import com.itextpdf.tool.xml.Experimental;

/**
 * <p>Creates pdf and site from an <a href="http://www.itextsupport.com/schema/itextdoc-1.0.xsd">itextdoc</a> XML file.
 * Please note, iText&reg; is under <strong>AGPLv3 license</strong>, see <a href="http://itextpdf.com/terms-of-use/">terms of use</a> for
 * more info.<p>
 * <p>This plugin <strong>requires Java 1.6</strong> or later</p>
 * @goal create
 * @description create pdf from itextdoc with help if iText and XMLWorker
 * @author redlab_b http://www.redlab.be/
 * @version 1.0.2
 * @threadSafe false
 * @requiresProject false
 * @phase package
 */
public class ItextDocMojo
		extends AbstractMojo {
	/**
	 * Location of the file.
	 *
	 * @parameter expression="${project.build.directory}/itextdoc" default "${project.build.directory}/itextdoc"
	 */
	private File out;
	/**
	 * The input folder.
	 *
	 * @parameter expression="${basedir}/src/itextdoc" default "${project.build.sourceDirectory}/itextdoc"
	 */
	private File src;
	/**
	 * The name of the itextdoc XML file, defaults to <code>doc.xml</code>.
	 * @parameter expression="doc.xml" default "doc.xml"
	 */
	private String docName;
	/**
	 * false if a jquery site shouldn't be created.
	 *
	 * @parameter expression="false" default false
	 */
	private boolean site;
	/**
	 * false if a flat site shouldn't be created.
	 *
	 * @parameter expression="false" default false
	 */
	private boolean flatsite;
	/**
	 * if you want a build to fail on error of doc creations, set to true.
	 *
	 * @parameter expression="false" default false
	 */
	private boolean failbuild;
	/**
	 * Set the encoding to use.
	 *
	 * @parameter expression="${project.build.sourceEncoding}" default "${project.build.sourceEncoding}"
	 */
	private String encoding;

	public void execute()
			throws MojoExecutionException {
		getLog().info("Running itextdoc.");
		try {

			File f = out;
			File in = src;
			if (!in.exists()) {
				throw new MojoExecutionException("Source not found in " + in);
			}
			if (!(new File(in, "doc.xml")).exists()) {
				throw new MojoExecutionException("Required doc.xml not found.");
			}
			if (!f.exists()) {
				if (!f.mkdirs()) {
					throw new MojoExecutionException("Unable to create target folder " + f.getPath());
				}
			}
			try {
				getLog().info(String.format("Processing itextdoc from %s to %s.", in.getPath(), f.getPath()));
				if (null == encoding) {
					encoding = Charset.defaultCharset().name();
					getLog().info(String.format("encoding not set, using %s the jvm default.", encoding));
				}
				Launcher l = new Launcher(this);
				l.parse(in, f, docName);
			} catch (IOException e) {
				throw new MojoExecutionException("Parsing Error", e);
			}
		} catch (MojoExecutionException e) {
			if (!failbuild) {
				getLog().info(e.getMessage(), e);
			} else {
				throw e;
			}
		}
	}

	/**
	 * @return true if an ajax site should be made, this is Experimental
	 */
	@Experimental("Still in development")
	public boolean getSite() {
		return site;
	}

	/**
	 * @return true if a flatsite should be made
	 */
	@Experimental("Still in development")
	public boolean getFlatSite() {
		return flatsite;
	}

	/**
	 * @return the encoding
	 */
	public String getEncoding() {
		return encoding;
	}

	/**
	 * @return the docName
	 */
	public String getDocName() {
		return docName;
	}


}
