/*
 *
 * This file is part of the iText (R) project. Copyright (c) 1998-2011 1T3XT BVBA Authors: Balder Van Camp, et al.
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU Affero General
 * Public License version 3 as published by the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY
 * 1T3XT, 1T3XT DISCLAIMS THE WARRANTY OF NON INFRINGEMENT OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
 * details. You should have received a copy of the GNU Affero General Public License along with this program; if not,
 * see http://www.gnu.org/licenses or write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA, 02110-1301 USA, or download the license from the following URL: http://itextpdf.com/terms-of-use/
 *
 * The interactive user interfaces in modified source and object code versions of this program must display Appropriate
 * Legal Notices, as required under Section 5 of the GNU Affero General Public License.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License, a covered work must retain the producer
 * line in every PDF that is created or manipulated using iText.
 *
 * You can be released from the requirements of the license by purchasing a commercial license. Buying such a license is
 * mandatory as soon as you develop commercial activities involving the iText software without disclosing the source
 * code of your own applications. These activities include: offering paid services to customers as an ASP, serving PDFs
 * on the fly in a web application, shipping iText with a closed source product.
 *
 * For more information, please contact iText Software Corp. at this address: sales@itextpdf.com
 */
package com.itextpdf.tool.itextdoc.codestyler;

import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;

import com.itextpdf.text.Element;
import com.itextpdf.text.log.Logger;
import com.itextpdf.text.log.LoggerFactory;
import com.itextpdf.tool.xml.Tag;
import com.itextpdf.tool.xml.WorkerContext;
import com.itextpdf.tool.xml.exceptions.RuntimeWorkerException;
import com.itextpdf.tool.xml.html.NonSanitizedTag;

/**
 * @author redlab
 *
 */
public class CodeStylerPreTagHandler extends NonSanitizedTag {

	/**
	 * the attribute of which the value is the coding language key to use on the pre tag.
	 */
	public static final String ATTRIBUTE = "code";
	private static final Logger LOG = LoggerFactory.getLogger(CodeStylerPreTagHandler.class);
	private final CodeStylerFactory factory;

	/**
	 *
	 */
	public CodeStylerPreTagHandler() {
		this.factory = new CodeStylerFactory();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.itextpdf.tool.xml.html.NonSanitizedTag#content(com.itextpdf.tool.xml.WorkerContext,
	 * com.itextpdf.tool.xml.Tag, java.lang.String)
	 */
	@Override
	public List<Element> content(final WorkerContext ctx, final Tag tag, final String content) {
		List<Element> l = new ArrayList<Element>(1);
		String lang = tag.getAttributes().get(ATTRIBUTE);
		if (null != lang && lang.length() > 0 ) {
			try {
				factory.getCodeStyler(lang, tag).apply(new StringReader(content), l);
				return l;
			} catch (IOException e) {
				throw new RuntimeWorkerException(e);
			} catch (InvalidCodeStyleException e) {
				LOG.warn(String
						.format("Requested Code Style not found for %s as coding language, proceeding with normal <pre> handling",
								lang));
				return super.content(ctx, tag, content);
			}
		} else {
			return super.content(ctx, tag, content);
		}
	}
}
