/*
 *
 * This file is part of the iText (R) project. Copyright (c) 1998-2011 1T3XT BVBA Authors: Balder Van Camp, et al.
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU Affero General
 * Public License version 3 as published by the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY
 * 1T3XT, 1T3XT DISCLAIMS THE WARRANTY OF NON INFRINGEMENT OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
 * details. You should have received a copy of the GNU Affero General Public License along with this program; if not,
 * see http://www.gnu.org/licenses or write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA, 02110-1301 USA, or download the license from the following URL: http://itextpdf.com/terms-of-use/
 *
 * The interactive user interfaces in modified source and object code versions of this program must display Appropriate
 * Legal Notices, as required under Section 5 of the GNU Affero General Public License.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License, a covered work must retain the producer
 * line in every PDF that is created or manipulated using iText.
 *
 * You can be released from the requirements of the license by purchasing a commercial license. Buying such a license is
 * mandatory as soon as you develop commercial activities involving the iText software without disclosing the source
 * code of your own applications. These activities include: offering paid services to customers as an ASP, serving PDFs
 * on the fly in a web application, shipping iText with a closed source product.
 *
 * For more information, please contact iText Software Corp. at this address: sales@itextpdf.com
 */
package com.itextpdf.tool.itextdoc.codestyler.java;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.itextpdf.text.Chunk;
import com.itextpdf.text.Element;
import com.itextpdf.tool.xml.Tag;
import com.itextpdf.tool.xml.css.apply.ChunkCssApplier;
import com.itextpdf.tool.xml.parser.State;

/**
 * @author redlab
 *
 */
public class JavaCodeStyleContext {
	private static Set<Character> separators = new HashSet<Character>(Arrays.asList(new Character[] { '{', '}', ';',
			'.', '\'', '(', ')', '!', '%', '+', '-', '*', '/', '<', '>', '&', '|', '=' }));
	private static Set<String> keywords = new HashSet<String>(Arrays.asList(new String[] { "abstract", "continue",
			"for", "new", "switch", "assert", "default", "goto", "package", "synchronized", "boolean", "do", "if",
			"private", "this", "break", "double", "implements", "protected", "throw", "byte", "else", "import",
			"public", "throws", "case", "enum", "instanceof", "return", "transient", "catch", "extends", "int",
			"short", "try", "char", "final", "interface", "static", "void", "class", "finally", "long", "strictfp",
			"volatile", "const", "float", "native", "super", "while" }));
	private final List<Element> target;
	private final StateController data;
	private final Tag tagNormal;
	private final Tag tagKeyword;
	private final Tag tagText;
	private final ChunkCssApplier css;
	private boolean intext;
	private char textchar;
	private final StringBuilder buffer;
	private char lastChar = '.';
	private final Tag tagjavadoc;
	private final Tag tagAnnotation;
	private final Tag tagComment;
	private JavaState javaState;
	private JavaState previousChunkState;
	private final StringBuilder internal;

	/**
	 * @param target
	 * @param t
	 */
	public JavaCodeStyleContext(final List<Element> target, final Tag t) {
		this.target = target;
		tagNormal = copyTag(t);
		tagKeyword = copyTag(t);
		tagKeyword.getCSS().put("color", "#880000");
		tagText = copyTag(t);
		tagText.getCSS().put("color", "#0022FF");
		tagjavadoc = copyTag(t);
		tagjavadoc.getCSS().put("color", "#7777FF");
		tagAnnotation = copyTag(t);
		tagAnnotation.getCSS().put("color", "#777777");
		tagComment = copyTag(t);
		tagComment.getCSS().put("color", "#009900");
		css = new ChunkCssApplier();
		buffer = new StringBuilder();
		internal = new StringBuilder();
		data = new StateController(this);
	}

	private Tag copyTag(final Tag t) {
		Tag copy = new Tag(t.getName(), t.getNameSpace());
		copy.setParent(t.getParent());
		copy.getChildren().addAll(t.getChildren());
		copy.getAttributes().putAll(t.getAttributes());
		copy.getCSS().putAll(t.getCSS());
		return copy;
	}

	/**
	 * @return
	 */
	public State getState() {
		return this.data.getCurrentState();
	}

	/**
	 * @param unknown
	 */
	public void setState(final JavaState key) {
		this.javaState = key;
		this.data.setCurrentState(key);
	}

	/**
	 * @param c
	 */
	public void setTextChar(final char c) {
		this.textchar = c;

	}

	/**
	 * @return
	 */
	public boolean inText() {
		return this.intext;
	}

	/**
	 * @param b
	 */
	public void isInText(final boolean b) {
		this.intext = b;
	}

	/**
	 * @return
	 */
	public JavaCodeStyleContext buffer(final char c) {
		buffer.append(c);
		return this;
	}

	/**
	 * @return
	 */
	public Set<Character> getSeparators() {
		return separators;
	}

	/**
	 * @return
	 */
	public Set<String> getKeywords() {
		return keywords;
	}

	/**
	 * @param unknown
	 * @return
	 */
	public Tag getTag(final JavaState state) {
		Tag t = null;
		switch (state) {
		case NORMAL:
			t = tagNormal;
			break;
		case TEXT:
			t = tagText;
			break;
		case KEYWORD:
			t = tagKeyword;
			break;
		case JAVADOC:
			t = tagjavadoc;
			break;
		case ANNOTATION:
			t = tagAnnotation;
			break;
		case MULTILINE_COMMENT:
		case SINGLE_COMMENT:
			t = tagComment;
			break;
		default:
			t = tagNormal;
		}
		return t;
	}

	/**
	 * @param new Chunk(string) the chunk
	 * @param state the JavaState
	 */
	private void add(final String str, final JavaState state) {
		flushIfNeeded(state);
		internal.append(str);
	}

	/**
	 * @param chunk
	 * @param state
	 */
	private void add(final Chunk chunk, final JavaState state) {
		target.add(css.apply(chunk, getTag(state)));
	}

	/**
	 * @param c
	 * @param unknown
	 */
	private void add(final char c, final JavaState state) {
		flushIfNeeded(state);
		internal.append(c);
		previousChunkState = state;
	}

	/**
	 * @param state
	 */
	private void flushIfNeeded(final JavaState state) {
		if (null != previousChunkState && !state.equals(previousChunkState)) {
			add(new Chunk(internal.toString()), previousChunkState);
			internal.setLength(0);
		}
		previousChunkState = state;
	}

	/**
	 * @param c
	 */
	public void setLastChar(final char c) {
		this.lastChar = c;
	}

	/**
	 * @return
	 */
	public char getTextEscape() {
		return '\\';
	}

	/**
	 * @return
	 */
	public char getLastChar() {
		return lastChar;
	}

	/**
	 * @return
	 */
	public char getTextChar() {
		return textchar;
	}

	/**
	 *
	 */
	private void clearBuffer() {
		buffer.setLength(0);
	}

	/**
	 * @return
	 */
	public String getBuffer() {
		return buffer.toString().replaceAll("\t", "    ");
	}

	/**
	 * @param unknown
	 */
	public void addBuffer(final JavaState state) {
		add(getBuffer(), state);
		clearBuffer();
	}

	/**
	 * @return
	 */
	public int bufferLength() {
		return buffer.length();
	}

	/**
	 *
	 */
	public void close() {
		if (null != previousChunkState) {
			if (buffer.length() > 0) {
				addBuffer(previousChunkState);
			}
			if (internal.length() > 0) {
				add(new Chunk(internal.toString().replaceAll("\t", "    ")), previousChunkState);
			}
		}
	}

	/**
	 * Delete i chars from the buffers end.
	 *
	 * @param i
	 */
	public void delete(final int i) {
		if (i > 0 && internal.length() - i >= 0) {
			internal.setLength(internal.length() - i);
		}
	}

	/**
	 * @return
	 */
	public JavaState getJavaState() {
		return javaState;
	}

}
