/**
 *
 */
package com.itextpdf.tool.itextdoc.codestyler.java;

import java.util.HashMap;
import java.util.Map;

import com.itextpdf.tool.xml.exceptions.RuntimeWorkerException;
import com.itextpdf.tool.xml.parser.State;

/**
 * @author  redlab
 */
public class StateController {
	/**
	 *
	 */
	private final Map<JavaState, State> hashMap;
	/**
	 *
	 */
	private State currentState;

	/**
	 *
	 */
	public StateController(final JavaCodeStyleContext ctx) {
		hashMap = new HashMap<JavaState, State>();
		hashMap.put(JavaState.ANNOTATION, new Annotation(ctx));
		hashMap.put(JavaState.JAVADOC, new JavadocState(ctx));
		hashMap.put(JavaState.LONGCOMMENT, new LongComment(ctx));
		hashMap.put(JavaState.MULTILINE_COMMENT, new MultiLineComment(ctx));
		hashMap.put(JavaState.NORMAL, new NormalState(ctx));
		hashMap.put(JavaState.SINGLE_COMMENT, new SingleComment(ctx));
		hashMap.put(JavaState.JAVADOC_ASTERIX, new JavadocAsterix(ctx));
		hashMap.put(JavaState.COMMENT, new Comment(ctx));
		hashMap.put(JavaState.TEXT, new TextState(ctx));
		hashMap.put(JavaState.MULTILINE_COMMENT_ASTERIX, new MultilineCommentAsterix(ctx));
	}


	/**
	 * @return the currentState
	 */
	public State getCurrentState() {
		return currentState;
	}

	/**
	 * @param key the currentState to set
	 */
	public void setCurrentState(final JavaState key) {
		this.currentState = hashMap.get(key);
		if (null == currentState) {
			throw new RuntimeWorkerException(String.format("No state with key %s", key));
		}
	}
}