/*
 *
 * This file is part of the iText (R) project. Copyright (c) 1998-2011 1T3XT BVBA Authors: Balder Van Camp, et al.
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU Affero General
 * Public License version 3 as published by the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY
 * 1T3XT, 1T3XT DISCLAIMS THE WARRANTY OF NON INFRINGEMENT OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
 * details. You should have received a copy of the GNU Affero General Public License along with this program; if not,
 * see http://www.gnu.org/licenses or write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA, 02110-1301 USA, or download the license from the following URL: http://itextpdf.com/terms-of-use/
 *
 * The interactive user interfaces in modified source and object code versions of this program must display Appropriate
 * Legal Notices, as required under Section 5 of the GNU Affero General Public License.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License, a covered work must retain the producer
 * line in every PDF that is created or manipulated using iText.
 *
 * You can be released from the requirements of the license by purchasing a commercial license. Buying such a license is
 * mandatory as soon as you develop commercial activities involving the iText software without disclosing the source
 * code of your own applications. These activities include: offering paid services to customers as an ASP, serving PDFs
 * on the fly in a web application, shipping iText with a closed source product.
 *
 * For more information, please contact iText Software Corp. at this address: sales@itextpdf.com
 */
package com.itextpdf.tool.itextdoc.pdf;

import java.io.FileNotFoundException;
import java.io.UnsupportedEncodingException;

import com.itextpdf.text.DocumentException;
import com.itextpdf.tool.xml.NoCustomContextException;
import com.itextpdf.tool.xml.Pipeline;
import com.itextpdf.tool.xml.PipelineException;
import com.itextpdf.tool.xml.ProcessObject;
import com.itextpdf.tool.xml.Tag;
import com.itextpdf.tool.xml.WorkerContext;
import com.itextpdf.tool.xml.pipeline.AbstractPipeline;

/**
 * @author redlab_b http://www.redlab.be/
 *
 */
public class DocumentationPipeline extends AbstractPipeline<DocumentationPipelineContext> {


	private final DocumentationConfigFactory factory;

	/**
	 * @param config the configuration.
	 * @param next the next pipeline
	 * @param core
	 */
	public DocumentationPipeline(final DocumentationConfigFactory factory, final Pipeline<?> next) {
		super(next);
		this.factory = factory;
	}

	/*
	 * (non-Javadoc)cbadb83ebaba23c29c4418b91116879b26d22d3a
	 *
	 * @see com.itextpdf.tool.xml.pipeline.AbstractPipeline#init(com.itextpdf.tool .xml.WorkerContext)
	 */
	@Override
	public Pipeline<?> init(final WorkerContext context) throws PipelineException {
		DocumentationPipelineContext ctx = new DocumentationPipelineContext();
		ctx.setResolver(factory.css());
		ctx.setConfig(factory.getConfig());
		ctx.setCore(factory.getCore());
		context.put(getContextKey(), ctx);
		return super.init(context);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.itextpdf.tool.xml.pipeline.AbstractPipeline#open(com.itextpdf.tool .xml.Tag,
	 * com.itextpdf.tool.xml.ProcessObject)
	 */
	@Override
	public Pipeline<?> open(final WorkerContext ctx, final Tag t, final ProcessObject po) throws PipelineException {
		final DocumentationPipelineContext dpc = getLocalContext(ctx);
		dpc.bookmarkIncrement();
		try {
			DocumentationPipelineCore actions = dpc.getCore();
			if (actions.isValidPart(t)) {
				actions.openValidPart( t, dpc);
			} else if (dpc.isIBookmarking() && null != t.getAttributes().get("ibookmark")) {
				actions.createIBookMark(dpc, t.getAttributes());
			} else if ("css".equalsIgnoreCase(t.getName())) {
				actions.openCss(t, dpc);
			} else if ("pdf".equals(t.getName())) {
				actions.openPdf(ctx, t, dpc);
			} else if ("imagedir".equalsIgnoreCase(t.getName())) {
				actions.setImageProvider(ctx, t);
			} else if ("rootlink".equalsIgnoreCase(t.getName())) {
				actions.setLinkProvider(ctx, t);
			} else if ("body".equalsIgnoreCase(t.getName())) {
				actions.openBody(t, dpc);
			} else if ("codepart".equalsIgnoreCase(t.getName())) {
				// add file with code styler
				actions.handleCodePart(t, po, dpc);
			}
		} catch (NoCustomContextException e) {
			throw new PipelineException(e);
		} catch (FileNotFoundException e) {
			throw new PipelineException(e);
		} catch (DocumentException e) {
			throw new PipelineException(e);
		} catch (UnsupportedEncodingException e) {
			throw new PipelineException(e);
		}
		return super.open(ctx, t, po);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.itextpdf.tool.xml.pipeline.AbstractPipeline#close(com.itextpdf.tool .xml.Tag,
	 * com.itextpdf.tool.xml.ProcessObject)
	 */
	@Override
	public Pipeline<?> close(final WorkerContext ctx, final Tag t, final ProcessObject po) throws PipelineException {
		DocumentationPipelineContext dpc = getLocalContext(ctx);
		dpc.bookmarkDecrement();
		DocumentationPipelineCore actions = dpc.getCore();
		if (actions.isValidPart(t)) {
			actions.closeValidPart( t, dpc);
		} else if (dpc.isIBookmarking() && null != t.getAttributes().get("ibookmark")) {
			actions.closeBookMark(dpc);
		} else if ("pdf".equals(t.getName())) {
			actions.closePdf(ctx);
		}
		return super.close(ctx, t, po);
	}
}
