/**
 * 
 */
package com.itextpdf.tool.itextdoc.pdf;

import java.io.FileNotFoundException;
import java.io.UnsupportedEncodingException;
import java.util.Map;

import com.itextpdf.text.DocumentException;
import com.itextpdf.tool.xml.NoCustomContextException;
import com.itextpdf.tool.xml.PipelineException;
import com.itextpdf.tool.xml.ProcessObject;
import com.itextpdf.tool.xml.Tag;
import com.itextpdf.tool.xml.WorkerContext;

/**
 * @author redlab
 *
 */
public interface DocumentationPipelineCore {

	/**
	 * Verifies that the tag is a part tag valid for PDF creation.
	 *
	 * @param t the Tag to check
	 * @return true if the tag is valid
	 */
	boolean isValidPart(final Tag t);

	/**
	 * @param t
	 * @param dpc
	 */
	void openBody(final Tag t, final DocumentationPipelineContext dpc);

	/**
	 * @param documentationPipeline TODO
	 * @param t
	 * @param dpc
	 * @throws PipelineException
	 */
	void openCss(final Tag t, final DocumentationPipelineContext dpc) throws PipelineException;

	/**
	 * @param ctx
	 * @param t
	 * @param dpc
	 * @throws NoCustomContextException
	 * @throws FileNotFoundException
	 * @throws DocumentException
	 */
	void openPdf(final WorkerContext ctx, final Tag t, final DocumentationPipelineContext dpc)
			throws NoCustomContextException, FileNotFoundException, DocumentException;

	/**
	 * @param documentationPipeline TODO
	 * @param t
	 * @param dpc
	 * @throws FileNotFoundException
	 * @throws UnsupportedEncodingException
	 */
	void openValidPart(final Tag t, final DocumentationPipelineContext dpc) throws FileNotFoundException,
			UnsupportedEncodingException;

	/**
	 * @param documentationPipeline TODO
	 * @param t
	 * @param dpc
	 */
	void closeValidPart(final Tag t, final DocumentationPipelineContext dpc);

	/**
	 * @param ctx
	 * @throws PipelineException
	 */
	void closePdf(final WorkerContext ctx) throws PipelineException;

	/**
	 * @param documentationPipeline TODO
	 * @param t
	 * @param po
	 * @param dpc
	 */
	void handleCodePart(final Tag t, final ProcessObject po, final DocumentationPipelineContext dpc);

	/**
	 * @param dpc
	 */
	void closeBookMark(final DocumentationPipelineContext dpc);

	/**
	 * @param dpc
	 * @param attributes
	 */
	void createIBookMark(final DocumentationPipelineContext dpc, final Map<String, String> attributes);

	/**
	 * @param ctx
	 * @param t
	 * @throws NoCustomContextException
	 */
	void setImageProvider(final WorkerContext ctx, final Tag t) throws NoCustomContextException;

	/**
	 * @param ctx
	 * @param t
	 * @throws NoCustomContextException
	 */
	void setLinkProvider(final WorkerContext ctx, final Tag t) throws NoCustomContextException;

}
