/*
 *
 * This file is part of the iText (R) project. Copyright (c) 1998-2011 1T3XT BVBA Authors: Balder Van Camp, et al.
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU Affero General
 * Public License version 3 as published by the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY
 * 1T3XT, 1T3XT DISCLAIMS THE WARRANTY OF NON INFRINGEMENT OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
 * details. You should have received a copy of the GNU Affero General Public License along with this program; if not,
 * see http://www.gnu.org/licenses or write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA, 02110-1301 USA, or download the license from the following URL: http://itextpdf.com/terms-of-use/
 *
 * The interactive user interfaces in modified source and object code versions of this program must display Appropriate
 * Legal Notices, as required under Section 5 of the GNU Affero General Public License.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License, a covered work must retain the producer
 * line in every PDF that is created or manipulated using iText.
 *
 * You can be released from the requirements of the license by purchasing a commercial license. Buying such a license is
 * mandatory as soon as you develop commercial activities involving the iText software without disclosing the source
 * code of your own applications. These activities include: offering paid services to customers as an ASP, serving PDFs
 * on the fly in a web application, shipping iText with a closed source product.
 *
 * For more information, please contact iText Software Corp. at this address: sales@itextpdf.com
 */
package com.itextpdf.tool.itextdoc.pdf;

import java.util.HashMap;
import java.util.Map;

import com.itextpdf.text.Image;
import com.itextpdf.text.pdf.PdfDestination;
import com.itextpdf.text.pdf.PdfOutline;
import com.itextpdf.tool.xml.NoCustomContextException;
import com.itextpdf.tool.xml.Tag;
import com.itextpdf.tool.xml.WorkerContext;
import com.itextpdf.tool.xml.html.HeaderNode;
import com.itextpdf.tool.xml.pipeline.html.HtmlPipeline;
import com.itextpdf.tool.xml.pipeline.html.HtmlPipelineContext;
import com.itextpdf.tool.xml.pipeline.html.ImageProvider;
import com.itextpdf.tool.xml.pipeline.html.LinkProvider;

/**
 * @author redlab_b http://www.redlab.be/
 *
 */
public class PdfUtilImpl implements PdfUtil {
	@Override
	public void setLinkProvider(final WorkerContext ctx, final Tag t) throws NoCustomContextException {
		final Map<String, String> attr = t.getAttributes();
		final HtmlPipelineContext hpc = (HtmlPipelineContext) ctx.get(HtmlPipeline.class.getName());
		hpc.setLinkProvider(new LinkProvider() {

			public String getLinkRoot() {
				return attr.get("url");
			}
		});
	}

	@Override
	public void setImageProvider(final WorkerContext ctx, final Tag t) throws NoCustomContextException {
		final Map<String, String> attr = t.getAttributes();
		final HtmlPipelineContext hpc = (HtmlPipelineContext) ctx.get(HtmlPipeline.class.getName());
		hpc.setImageProvider(new ImageProvider() {
			private final HashMap<String, Image> map = new HashMap<String, Image>();

			public void store(final String src, final Image img) {
				map.put(src, img);
			}

			public Image retrieve(final String src) {
				return map.get(src);
			}

			public void reset() {
				map.clear();
			}

			public String getImageRootPath() {
				return attr.get("dir");
			}
		});
	}

	@Override
	public void closeBookMark(final DocumentationPipelineContext dpc) {
		dpc.setOutline(dpc.getOutline().parent());
	}

	@Override
	public void createIBookMark(final DocumentationPipelineContext dpc, final Map<String, String> attr) {
		String title = attr.get("ibookmark");
		HeaderNode master;
		if (null != dpc.getIBookmarkParent()) {
			master = dpc.getIBookmarkParent();
		} else {
			master = dpc.getOutline();
		}
		createPDFBookmark(dpc, title, master);
	}

	@Override
	public void createBookMark(final DocumentationPipelineContext dpc, final Map<String, String> attr) {
		String title = attr.get("title");
		if (null == title) {
			title = "!untitled!";
		}
		HeaderNode node = createPDFBookmark(dpc, title, dpc.getOutline());
		dpc.setIBookmarkParent(node);
	}

	@Override
	public HeaderNode createPDFBookmark(final DocumentationPipelineContext dpc, final String title,
			final HeaderNode master) {
		int level = dpc.bookmarkLevel();
		if (null != dpc.getIBookmarkParent()) {
			dpc.setIBookmarkParent(null);
		}
		HeaderNode root = null;
		if (level == master.level()) {
			root = master.parent();
		} else if (level < master.level()) {
			root = master.parent();
			while (level < (root.level())) {
				root = root.parent();
			}
		} else {
			root = master;
		}
		PdfDestination destination = new PdfDestination(PdfDestination.XYZ, 20,
				dpc.getWriter().getVerticalPosition(false), 0);
		HeaderNode node = new HeaderNode(level, new PdfOutline(root.outline(), destination, title), root);
		dpc.setOutline(node);
		return node;
	}

}
